@@ -1,6 +1,7 @@
 #include <asm/unistd.h>
 #include <errno.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <signal.h>
 #include "vm86.h"
 #include "harness.h"
@@ -11,8 +12,23 @@ static int v86_debugging = 0;
 /* The purpose of this file is to interact with the kernel and run the
    V86 task.  */
 
-static
-_syscall2(int, vm86, unsigned long, fn, struct vm86plus_struct *, v86);
+/* Some versions of libc don't have vm86, so call the system call
+ * directly. */
+static int vm86(unsigned long fn, struct vm86plus_struct *v86)
+{
+  int res;
+  __asm__ __volatile__("int $0x80" :
+		       "=a" (res) :
+		       "0" (__NR_vm86),
+		       "b" (fn),
+		       "c" (v86) :
+		       "memory");
+  if(res < 0) {
+    errno = -res;
+    return -1;
+  }
+  return res;
+}
 
 static void vm86_dump(void)
 {
@@ -75,6 +91,10 @@ void vm86_run()
     new_segv.sa_flags = SA_ONESHOT | SA_NOMASK;
     sigaction(SIGSEGV, &new_segv, &old_segv);
     res = vm86(VM86_ENTER, &v86);
+    if (res == -1) {
+      perror("vm86");
+      exit(1);
+    }
     sigaction(SIGSEGV, &old_segv, NULL);
 
     if (v86_debugging)
